import React, {Component} from 'react';
import R from 'ramda';
import moment from 'moment';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import RU from '../../utils/ramda-utils';
import {ERROR, WARNING} from '../../utils/ci-utils';
import {ComponentNavigator} from '../playground/component-navigator/component-navigator';
import {XlReactDipDynamicForm} from '../../xl-react-form/xl-react-dip-dynamic-form';
import KitchenTestComponent from '../playground/kitchen-test-component';
import {getHandlers, lookupSuggestedElements} from '../handlers/predefined-handlers';
import {formInitialState} from './form-initial-state';

export class Form extends Component {
    static propTypes = {match: PropTypes.object};

    state = formInitialState;

    /* eslint-enable sort-keys */
    findPropertyByName = (formName, propertyName) =>
        R.find(R.propEq('name', propertyName))(this.state[formName].metadata.properties);
    /* eslint-disable sort-keys */

    time = moment({
        year: 2016, month: 10, day: 9, hours: 15, minutes: 10, seconds: 0, milliseconds: 0
    }).toDate();

    render() {
        const {match} = this.props;

        R.forEach((formName) => {
            this.findPropertyByName(formName, 'datetimeName').default = this.time;
        })(['dynamicFormEmpty', 'dynamicFormDefault']);

        const updateState = (modelName, newState) => {
            const updatedState = {
                ...newState,
                model: {...R.mergeRight(this.state[modelName].model, newState.model)}
            };
            this.setState({...this.state, [modelName]: {...updatedState}});
            return updatedState;
        };

        const formHandlers = {
            CI: getHandlers(),
            LIST_OF_CI: getHandlers(),
            LIST_OF_STRING: {nameComponentList: getHandlers()},
            LOOKUP_CI: getHandlers(lookupSuggestedElements),
            SET_OF_CI: getHandlers()
        };

        const warningValidator = (fieldName) => R.pipe(
            R.propOr([], 'validation-messages'),
            R.find(R.where({
                type: R.either(R.equals(WARNING), R.equals(ERROR)),
                property: R.equals(fieldName)
            })),
            R.cond([
                [R.isNil, R.always({})],
                [R.T, R.pipe(
                    R.pick(['text', 'type']),
                    R.assoc('show', false),
                    R.objOf('message'),
                    R.assoc('condition', true))]
            ]));

        const vaultValidator = (fieldName, model) => {
            const isApplicable = () => R.equals(fieldName, 'vault') && RU.isDefined(model.provider);

            if (isApplicable()) {
                if (!R.pipe(
                    R.pluck('id'),
                    R.contains(model.provider)
                )(lookupSuggestedElements)) {
                    return {
                        condition: true,
                        message: {show: false, text: 'Invalid vault provider value', type: WARNING}
                    };
                }
            }

            return {condition: false};
        };

        const formValidator = (formState) => (fieldName) =>
            warningValidator(fieldName)(formState);

        const xlReactDipForm = <XlReactDipDynamicForm
            dateTimeFormat={'YYYY-MM-DDTHH:mm:ss.SSS'}
            handlers={formHandlers} objLabel={'id'}
        />;
        const xlReactDipFormWithWarnings = <XlReactDipDynamicForm
            handlers={formHandlers} objLabel={'id'}
            validators={[formValidator(formInitialState.dynamicFormEmptyWithWarnings)]}
        />;

        const xlReactDipFormWithLookup = <XlReactDipDynamicForm
            handlers={formHandlers} objLabel={'id'}
            validators={[formValidator(formInitialState.dynamicWithLookup), vaultValidator]}
        />;

        const forms = {
            dynamicFormEmpty: <KitchenTestComponent
                className="xl-react-dip-dynamic-form-empty"
                currentState={this.state}
                disabledToggles={['auto-focus']}
                key={'dynamicFormEmpty'}
                label={'Empty form'}
                modelName={'dynamicFormEmpty'}
                updateState={updateState}>
                {xlReactDipForm}
            </KitchenTestComponent>,
            dynamicFormEmptyWithWarnings: <KitchenTestComponent
                className="xl-dip-dynamic-form-empty"
                currentState={this.state}
                disabledToggles={['auto-focus']}
                key={'dynamicFormEmptyWithWarnings'}
                label={'Empty form with warnings'}
                modelName={'dynamicFormEmptyWithWarnings'}
                updateState={updateState}>
                {xlReactDipFormWithWarnings}
            </KitchenTestComponent>,
            dynamicFormDefault: <KitchenTestComponent
                className="xl-react-dip-dynamic-form-default"
                currentState={this.state}
                disabledToggles={['auto-focus']}
                key={'dynamicFormDefault'}
                label={'Form with default values'}
                modelName={'dynamicFormDefault'}
                updateState={updateState}>
                {xlReactDipForm}
            </KitchenTestComponent>,
            dynamicFormAsContainment: <KitchenTestComponent
                className="xl-react-dip-dynamic-form-as-containment"
                currentState={this.state}
                disabledToggles={['auto-focus']}
                key={'dynamicFormAsContainment'}
                label={'Form with an asContainment property'}
                modelName={'dynamicFormAsContainment'}
                updateState={updateState}>
                {xlReactDipForm}
            </KitchenTestComponent>,
            dynamicFormWithMultipleCategories: <KitchenTestComponent
                className="xl-react-dip-dynamic-form-with-multiple-categories"
                currentState={this.state}
                disabledToggles={['auto-focus']}
                key={'dynamicFormWithMultipleCategories'}
                label={'Form with multiple categories'}
                modelName={'dynamicFormWithMultipleCategories'}
                updateState={updateState}>
                {xlReactDipForm}
            </KitchenTestComponent>,
            dynamicWithLookup: <KitchenTestComponent
                className="xl-react-dip-dynamic-form-with-lookup-values"
                currentState={this.state}
                disabledToggles={['auto-focus']}
                key={'dynamicWithLookup'}
                label={'Form with lookup values'}
                modelName={'dynamicWithLookup'}
                updateState={updateState}>
                {xlReactDipFormWithLookup}
            </KitchenTestComponent>,
            dynamicReadonlyForm: <KitchenTestComponent
                className="xl-react-dip-form-readonly"
                currentState={this.state}
                disabledToggles={['auto-focus']}
                key={'dynamicReadonlyForm'}
                label={'Readonly Form'}
                modelName={'dynamicReadonlyForm'}
                updateState={updateState}>
                {xlReactDipForm}
            </KitchenTestComponent>
        };

        const isSingleItem = match && forms[match.params.name];

        return (
            <div className={classNames('form-page', {'single-item': isSingleItem})}>
                <ComponentNavigator componentType="form" state={this.state}/>
                {isSingleItem ? forms[match.params.name] : R.values(forms)}
            </div>
        );
    }
}
