import R from 'ramda';
import RS from 'ramdasauce';
import RU from '../utils/ramda-utils';
import {isDisabled} from '../utils/components-utils';
import {defaultValidatorForRequiredField} from '../validators/default-validator-for-required-field';
import {ERROR, WARNING} from '../utils/ci-utils';

/* eslint-disable complexity */
const isToShowMessage = (props, pristine, validationToShow = {message: {show: false}}) => {
    if (props.showParentMessages) {
        return false;
    }

    const isTypeDefined = R.pipe(
        R.prop('type'),
        R.complement(R.isNil)
    );

    const isGenerallyEnabledMessage = !isTypeDefined(props.message) && R.pathOr(false, ['message', 'show'], props);

    const isEnabled = (message) => message.show || isGenerallyEnabledMessage;

    const showClientSideMessage = (message) =>
        (message.rule && message.type === WARNING || message.type === ERROR) && (!pristine || isEnabled(message));

    const showServerSideMessage = (message) => !message.rule && message.type === WARNING;

    const showAnyMessage = (message) => showClientSideMessage(message) || showServerSideMessage(message);

    const shouldBeShown =
        R.pathSatisfies((message) => isTypeDefined(message) && showAnyMessage(message), ['message']);

    return R.any(shouldBeShown)([props, validationToShow]);
};

export const validateComponent = (props, validators, model, pristine) => {
    if (isDisabled(props.disabled, props.displayMode)) {
        return {valid: true};
    }
    if (R.pathSatisfies((text) => !RS.isNilOrEmpty(text), ['message', 'text'], props)) {
        return {
            message: {
                ...props.message,
                show: isToShowMessage(props, pristine)
            },
            valid: props.message.type !== ERROR
        };
    }

    const evaluatedValidations = R.map((validator) => validator(model), validators);
    const validationToShow = R.find((result) => result.condition, evaluatedValidations);
    return R.whereEq({condition: true}, validationToShow || {}) ?
        {
            message: {...validationToShow.message, show: isToShowMessage(props, pristine, validationToShow)},
            valid: validationToShow.message.type !== ERROR
        } :
        {valid: true};
};

export const invokeCallback = (props, data) => {
    if (R.is(Function, data)) {
        data();
    } else {
        RU.invoke(data.functionName, data.values, props);
    }
};

export const computeNextState = (context, props, validators, model, {
    allowUndefinedModel = false,
    invokeCallbackEnabled = true,
    customCallbacks = [],
    pristine = false
} = {}) => {
    const prevState = context.state;
    const {message = props.message, valid} = validateComponent(props, validators, model, pristine);
    const newState = {};
    let dataForCallbacks = [...customCallbacks];

    if (prevState.valid !== valid) {
        newState.valid = valid;
        dataForCallbacks = [...dataForCallbacks, {functionName: 'onValidityChange', values: [valid]}];
    }

    if (!R.equals(prevState.message, message)) {
        newState.message = message;
        dataForCallbacks = [...dataForCallbacks, {
            functionName: 'onMessageChange',
            values: [message]
        }, {
            functionName: 'messageHandler',
            values: [message]
        }];
    }

    if (prevState.pristine !== pristine) {
        newState.pristine = pristine;
        dataForCallbacks = [...dataForCallbacks, {functionName: 'onPristinityChange', values: [pristine]}];
    }
    const isModelDefined = allowUndefinedModel || !R.isNil(model);

    if (isModelDefined && !R.equals(prevState.model, model)) {
        newState.model = model;
        dataForCallbacks = [...dataForCallbacks, {
            functionName: 'onModelChange',
            values: [model, pristine]
        }];
    }

    context.setState(newState, () => {
        if (invokeCallbackEnabled) {
            R.forEach((data) => invokeCallback(props, data), dataForCallbacks);
        }
    });
};

export const computeOnChangeState = (context, props, validators, model, config = {}) =>
    computeNextState(context, props, validators, model, {...config, allowUndefinedModel: true});

export const computeNextPropsState = (context, props, state, validators, invokeCallbackEnabled = true) => {
    const model = !R.isNil(props.model) ? props.model : state.model;
    if (!R.isNil(model)) {
        computeNextState(context, props, validators, model, {
            invokeCallbackEnabled,
            pristine: state.pristine
        });
    }
};

export const computeInitialState = (context, props, validators, model, invokeCallbackEnabled = true) => {
    if (!R.isNil(model)) {
        computeNextState(context, props, validators, model, {
            customCallbacks: context.api ? [{functionName: 'register', values: [context.api]}] : [],
            invokeCallbackEnabled,
            pristine: true
        });
    }
};

export const getDefaultValidatorForRequiredField = (props) => {
    const config = props.metadata ? props.metadata : props.options;
    return defaultValidatorForRequiredField(
        config.required || config.asRequired,
        config.requiredText
    );
};

export const getValidators = (props, customValidators = []) =>
    [
        getDefaultValidatorForRequiredField(props),
        ...customValidators,
        ...props.validators
    ];

/* eslint-enable complexity */
