import R from 'ramda';
import moment from 'moment';
import RU from './ramda-utils';

export const PRINT_FORMAT = 'MMMM Do YYYY, h:mm:ss a';
export const isFile = (value) => value.toString() === '[object File]';

export const printValue = (value) =>
    R.cond([
        [R.isNil, R.always(null)],
        [moment.isMoment, () => value.format(PRINT_FORMAT)],
        [R.is(Date), () => moment(value)
            .format(PRINT_FORMAT)],
        [isFile, (file) => file.name],
        [R.is(Object), JSON.stringify],
        [R.T, () => value.toString()]
    ])(value);

export const hasValue = (prop) => {
    if (R.is(String, prop)) {
        return !R.isEmpty(prop);
    }
    return !R.isNil(prop) && !R.isEmpty(prop);
};

export const getDefaultValue = (props) => R.path(['metadata', 'default'], props);

export const getModelValue = (props, propName, fallbackValue) => {
    const getPropsValue = () => {
        const propValue = R.path([propName], props);
        return hasValue(propValue) ? propValue : getDefaultValue(props);
    };

    const value = getPropsValue();
    const modelValue = hasValue(value) ? value : fallbackValue;
    return R.clone(modelValue);
};

export const isDisabled = (disabled, displayMode) => disabled || displayMode === 'visual';

export const isComponentDisabled = ({disabled, displayMode}) => disabled || displayMode === 'visual';

export const resetPristinity = (context, props, config = {}, callback = R.F) => {
    if (!config.silent) {
        RU.invoke('onPristinityChange', [true], props);
    }
    context.state.pristine = true;
    context.setState({pristine: true});
    callback();
};

export const calculateMenuHeight = (maxMenuHeight, optionsNumber, optionsHeight, shouldCheckSubActions) => {
    const subActionsHeight = shouldCheckSubActions ? 30 : 0;
    const optionsHeightCombined = optionsNumber * optionsHeight;
    if (optionsHeightCombined > maxMenuHeight) {
        return maxMenuHeight + subActionsHeight;
    }
    return optionsHeightCombined + subActionsHeight;
};

const getParents = (node, ps) => {
    if (node.parentNode === null) {
        return ps;
    }
    return getParents(node.parentNode, ps.concat([node]));
};

const style = (node, prop) => window.getComputedStyle(node, null).getPropertyValue(prop);

const overflow = (node) => style(node, 'overflow') + style(node, 'overflow-y') + style(node, 'overflow-x');

const isScrollableNode = (node) => (/(auto|scroll)/).test(overflow(node));

export const getScrollParent = (node) => {
    if (!(node instanceof window.HTMLElement || node instanceof window.SVGElement)) {
        return undefined;
    }

    const ps = getParents(node.parentNode, []);
    const scrollElement = (elNode) => elNode || document.scrollingElement || document.documentElement;
    return R.pipe(R.find(isScrollableNode), scrollElement)(ps);
};

export const shouldDisplayDropdownUp = (inputElement, containerHeight) => { // eslint-disable-line complexity
    const parentScrollableContainer = getScrollParent(inputElement);

    if (R.isNil(inputElement) || R.isNil(parentScrollableContainer)) {
        return false;
    }
    const {top: parentTop, bottom: parentBottom} = parentScrollableContainer.getBoundingClientRect();
    const spaceDown = Math.min(parentBottom, window.innerHeight);
    const limitUp = Math.abs(parentTop) > window.innerHeight ? 0 : parentTop;
    const {bottom: bottomPosition, top: topPosition} = inputElement.getBoundingClientRect();
    const isEnoughSpaceDown = containerHeight + bottomPosition < spaceDown;
    const isEnoughSpaceUp = Math.abs(limitUp - topPosition) > containerHeight;
    return !isEnoughSpaceDown && isEnoughSpaceUp;
};
