import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {Table, AutoSizer, SortDirection} from 'react-virtualized';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {isError} from '../../utils/ci-utils';
import {
    MAX_ROWS,
    HEADER_HEIGHT,
    ROW_HEIGHT,
    getRowClassNames,
    calculateMapTableHeight,
    rowGetter,
    fillDataWithBlankRows,
    rowRenderer,
    COLUMNS
} from './map-table-utils';
import {createHeaderRowRenderer} from './header-row-renderer';
import {tableKeyColumn} from './columns/table-key-column';
import {tableValueColumn} from './columns/table-value-column';
import 'react-virtualized/styles.css';
import {MapKeyInputType} from './xl-react-dip-map-string-string-constants';

export default class MapTable extends Component {
    static propTypes = {
        assignRef: PropTypes.func.isRequired,
        assignTableRef: PropTypes.func.isRequired,
        cellState: PropTypes.shape({
            column: PropTypes.oneOf([COLUMNS.key, COLUMNS.value]),
            row: PropTypes.number.isRequired,
            value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired
        }),
        data: PropTypes.arrayOf(PropTypes.object.isRequired).isRequired,
        disabled: PropTypes.bool.isRequired,
        handleCloseButton: PropTypes.func.isRequired,
        handleKeyBlur: PropTypes.func.isRequired,
        handleNewKeyBlur: PropTypes.func.isRequired,
        handleNewKeyChange: PropTypes.func.isRequired,
        handleNewKeyFocus: PropTypes.func.isRequired,
        handleNewValueBlur: PropTypes.func.isRequired,
        handleNewValueFocus: PropTypes.func.isRequired,
        handleNewValueKeyDown: PropTypes.func.isRequired,
        handleRemoveKey: PropTypes.func.isRequired,
        handleValueChange: PropTypes.func.isRequired,
        hasKeyError: PropTypes.bool.isRequired,
        inputType: PropTypes.oneOf([MapKeyInputType.password, MapKeyInputType.text]).isRequired,
        message: xlMessageTypeProp,
        model: PropTypes.object.isRequired,
        newKey: PropTypes.string.isRequired,
        newValue: PropTypes.string.isRequired,
        noRowsRenderer: PropTypes.oneOfType([PropTypes.func, PropTypes.string]).isRequired,
        onSortChange: PropTypes.func.isRequired,
        showNewKeyValueRow: PropTypes.bool.isRequired,
        showRemoveButton: PropTypes.bool.isRequired,
        sortBy: PropTypes.oneOf([COLUMNS.key, COLUMNS.value]),
        sortDirection: PropTypes.oneOf([SortDirection.ASC, SortDirection.DESC]),
        updateCellState: PropTypes.func.isRequired,
        valid: PropTypes.bool.isRequired
    };

    state = {
        widths: {
            key: 0.5,
            value: 0.5
        }
    };

    resizeColumn = (tableWidth) => ({deltaX, leftColumn, rightColumn}) => this.setState(
        (currentState) => {
            const currentWidths = currentState.widths;
            const delta = deltaX / tableWidth;
            const leftColWidthPercent = currentWidths[leftColumn] + delta;
            const rightWidthPercent = currentWidths[rightColumn] - delta;

            return {
                widths: {
                    [leftColumn]: leftColWidthPercent,
                    [rightColumn]: rightWidthPercent
                }
            };
        }
    );

    render() {
        const {
            data,
            disabled,
            noRowsRenderer,
            onSortChange,
            sortBy,
            sortDirection,
            assignTableRef,
            handleCloseButton,
            handleNewValueBlur,
            handleNewValueFocus,
            handleNewValueKeyDown,
            handleRemoveKey,
            handleValueChange,
            inputType,
            message,
            model,
            newValue,
            showNewKeyValueRow,
            showRemoveButton,
            handleKeyBlur,
            hasKeyError,
            assignRef,
            handleNewKeyBlur,
            handleNewKeyChange,
            handleNewKeyFocus,
            newKey,
            valid,
            cellState,
            updateCellState
        } = this.props;
        const {widths} = this.state;

        const filledRowsData = fillDataWithBlankRows(data);
        const tableHeight = calculateMapTableHeight(MAX_ROWS, ROW_HEIGHT, HEADER_HEIGHT);
        return (
            <AutoSizer disableHeight={true} disableWidth={false}>
                {({width}) =>
                    <Table
                        className="-striped -highlight table-scrollable-container xl-virtualized-table"
                        data={filledRowsData}
                        gridClassName="xl-virtualized-table-body"
                        headerClassName="xl-virtualized-table-header-cell"
                        headerHeight={HEADER_HEIGHT}
                        headerRowRenderer={createHeaderRowRenderer(tableHeight, width, this.resizeColumn(width))}
                        height={tableHeight}
                        noRowsRenderer={noRowsRenderer}
                        ref={assignTableRef}
                        rowClassName={getRowClassNames}
                        rowCount={filledRowsData.length}
                        rowGetter={rowGetter(filledRowsData)}
                        rowHeight={ROW_HEIGHT}
                        rowRenderer={rowRenderer}
                        sort={onSortChange}
                        sortBy={sortBy}
                        sortDirection={sortDirection}
                        width={width}>
                        {tableKeyColumn({
                            assignRef,
                            cellState,
                            columnWidth: widths.key * width,
                            disabled,
                            handleKeyBlur,
                            handleNewKeyBlur,
                            handleNewKeyChange,
                            handleNewKeyFocus,
                            hasKeyError,
                            invalid: isError(valid, message) || hasKeyError,
                            newKey,
                            showNewKeyValueRow,
                            updateCellState
                        })}
                        {tableValueColumn({
                            cellState,
                            columnWidth: widths.value * width,
                            disabled,
                            handleCloseButton,
                            handleNewValueBlur,
                            handleNewValueFocus,
                            handleNewValueKeyDown,
                            handleRemoveKey,
                            handleValueChange,
                            inputType,
                            model,
                            newValue,
                            showNewKeyValueRow,
                            showRemoveButton,
                            updateCellState
                        })}
                    </Table>
                }
            </AutoSizer>
        );
    }
}
