// eslint-disable-line max-lines
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import $ from 'jquery';
import {getModelValue, isDisabled, resetPristinity} from '../../utils/components-utils';
import {
    computeInitialState,
    computeNextState,
    computeNextPropsState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import RU from '../../utils/ramda-utils';
import {computeNewStateOnNewKeyBlur, isKeyUnique} from '../input/input-service';
import {XlReactDipMapStringStringComponent} from './xl-react-dip-map-string-string-component';
import * as service from './xl-react-dip-map-string-string-service';
import {COLUMNS, getUpdatedCellState} from './map-table-utils';

const delayTime = 200;

const resetAllErrors = {
    duplicateKey: undefined,
    emptyKey: undefined,
    hasNewKeyError: undefined
};

export class XlReactDipMapStringString extends Component {
    static propTypes = {
        model: PropTypes.object,
        showNewKeyValueRow: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);

        this.state = {
            modelKeys: service.getModelKeys(props.model, true),
            newKey: '',
            newValue: '',
            searchWord: '',
            showNewKeyValueRow: false
        };
    }

    componentWillMount() {
        const model = getModelValue(this.props, 'model', {});
        computeInitialState(this, this.props, getValidators(this.props), model);
    }

    componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps));
        if (this.state.pristine) {
            const modelKeys = service.getModelKeys(this.state.model, true);
            this.setState({modelKeys});
        }
    }

    handleCleaningSearchWord = () => {
        this.setState({searchWord: ''});
    };

    /* eslint-disable complexity */
    handleKeyBlur = (changedKey, oldKey, input) => {
        const {model, modelKeys} = this.state;
        if (!R.has(oldKey, model)) {
            return;
        }

        const trimmedChangedKey = R.trim(changedKey);
        const hasKeyChanged = !R.equals(trimmedChangedKey, oldKey);

        const handleChangedKey = () => {
            if (!trimmedChangedKey) {
                this.setState({
                    emptyKey: true,
                    model: R.clone(model)
                });
                RU.invoke('focus', [], input);
            } else if (!isKeyUnique(trimmedChangedKey, oldKey)(model)) {
                const newState = {
                    duplicateKey: trimmedChangedKey,
                    model: R.clone(model)
                };
                this.setState(newState);
                RU.invoke('focus', [], input);
            } else {
                const updatedModel = service.getModelWithUpdatedKey(model, oldKey, trimmedChangedKey);
                const updatedModelKeys = service.getModelKeysWithUpdatedKey(modelKeys, oldKey, trimmedChangedKey);
                this.setState({modelKeys: updatedModelKeys});

                const focusToInputField = () => {
                    if (this.state.hasNewKeyError) {
                        this.delayedKeyOnInputFocus();
                    } else {
                        this.focusKeyInput(trimmedChangedKey);
                    }
                };
                this.applyNewState({
                    ...this.state,
                    model: updatedModel
                }, focusToInputField);
            }
        };

        if (!hasKeyChanged) {
            if (this.state.hasNewKeyError) {
                this.delayedKeyOnInputFocus();
            } else {
                this.setState(resetAllErrors);
            }
        } else {
            handleChangedKey();
        }
    };
    /* eslint-enable complexity */

    handleRemoveKey = (key) => {
        const model = R.dissoc(key, this.state.model);
        const modelKeys = R.without([key], this.state.modelKeys);
        this.setState({modelKeys});
        this.applyNewState({
            ...this.state,
            model
        }, this.focusOnInputKeyWithError);
    };

    handleSearchChanged = (searchWord) => {
        this.setState({searchWord});
    };

    handleSortChange = ({sortBy, sortDirection}) => {
        const {cellState, model} = this.state;
        this.setState({
            cellState: getUpdatedCellState(model, cellState, {sortBy, sortDirection}),
            sorting: {sortBy, sortDirection}
        });
    };

    handleValueChange = (key, newValue) => {
        if (R.has(key, this.state.model)) {
            this.applyNewState(R.assocPath(['model', key], newValue, this.state), this.focusOnInputKeyWithError);
        }
    };

    onCloseButton = () => {
        this.setState({
            ...resetAllErrors,
            newKey: '',
            newValue: '',
            showNewKeyValueRow: false
        });
    };

    onAddRow = () => setTimeout(() => {
        if (!this.state.hasNewKeyError) {
            this.setState({
                cellState: {
                    column: COLUMNS.key,
                    row: 0,
                    value: ''
                },
                searchWord: '',
                showNewKeyValueRow: true
            }, () => {
                this.tableRef.scrollToRow(0);
                this.delayedKeyOnInputFocus();
            });
        }
    }, delayTime);

    onNewKeyBlurDelayedHandler = () => {
        const newState = computeNewStateOnNewKeyBlur(this.state);
        const applyModelIfPossible = () => {
            const {newKey, newValueFocused} = this.state;

            if (!newValueFocused && R.trim(newKey)) {
                this.addModelKey();
            }
        };
        this.setState({...newState}, applyModelIfPossible);
    };

    onNewKeyBlur = () => setTimeout(this.onNewKeyBlurDelayedHandler, delayTime);

    onNewKeyChange = (event) => {
        this.setState({newKey: event.target.value});
    };

    onNewKeyFocus = () => {
        this.setState(resetAllErrors);
    };

    onNewValueBlur = () => {
        this.setState({newValueFocused: false});

        const {hasNewKeyError, newKey} = this.state;
        if (hasNewKeyError) {
            this.delayedKeyOnInputFocus();
        } else if (R.trim(newKey)) {
            this.addModelKey();
        }
    };

    onNewValueFocus = () => {
        this.setState({newValueFocused: true});
    };

    onNewValueKeyDown = ({target: {value}}) => {
        this.setState({newValue: value});
    };

    delayedKeyOnInputFocus = () => setTimeout(() => {
        this.inputKeyField.focus();
    }, delayTime);

    addModelKey = () => {
        const {newKey, newValue, model, modelKeys} = this.state;
        const keyIsUnique = isKeyUnique(newKey)(model);
        if (keyIsUnique) {
            this.setState({
                modelKeys: [newKey, ...modelKeys],
                newKey: '',
                newValue: '',
                showNewKeyValueRow: false
            });
            const focusOnField = () => {
                this.focusKeyInput(newKey);
            };
            this.applyNewState({
                model: {
                    ...model,
                    [newKey]: newValue
                }
            }, focusOnField);
        } else {
            this.setState({
                duplicateKey: newKey,
                emptyKey: undefined,
                hasNewKeyError: true
            });
        }
    };

    focusKeyInput = (key) => {
        const tableInputs = R.values($(this.tableWrapperRef)
            .find('.key-div input'));
        const mapKeyInputField = R.find(R.propSatisfies(R.equals(key), 'defaultValue'), tableInputs);
        if (mapKeyInputField) {
            mapKeyInputField.focus();
        }
    };

    focusOnInputKeyWithError = () => {
        if (this.state.hasNewKeyError) {
            this.delayedKeyOnInputFocus();
        }
    };

    api = {
        focus: () => {
            this.setState({showNewKeyValueRow: true}, () => {
                this.delayedKeyOnInputFocus();
            });
        },
        resetPristinity: (config) => resetPristinity(this, this.props, config)
    };

    applyNewState = ({duplicateKey, hasNewKeyError, model}, callback) => {
        const customCallbacks = callback ? [callback] : [];
        this.setState({
            duplicateKey,
            emptyKey: undefined,
            hasNewKeyError
        });
        computeNextState(this, this.props, getValidators(this.props), model, {customCallbacks});
    };

    assignRef = (input) => {
        this.inputKeyField = input;
    };

    updateCellState = (cellState) => {
        this.setState({cellState});
    };

    assignTableRef = (table) => {
        this.tableRef = table;
    };

    assignTableWrapperRef = (tableWrapper) => {
        this.tableWrapperRef = tableWrapper;
    };

    render() {
        const {displayMode, metadata, onValidityChange} = this.props;
        const {
            message,
            cellState,
            duplicateKey,
            emptyKey,
            model,
            modelKeys,
            newKey,
            newValue,
            pristine,
            showNewKeyValueRow,
            searchWord,
            valid,
            sorting
        } = this.state;
        const {autoHide, description, label, password, required} = metadata;
        const disabled = isDisabled(this.props.disabled, displayMode);

        return <XlReactDipMapStringStringComponent {...({
            assignRef: this.assignRef,
            assignTableRef: this.assignTableRef,
            assignTableWrapperRef: this.assignTableWrapperRef,
            autoHide,
            cellState,
            description,
            disabled,
            duplicateKey,
            emptyKey,
            encrypted: password,
            handleAddRow: this.onAddRow,
            handleCloseButton: this.onCloseButton,
            handleNewKeyBlur: this.onNewKeyBlur,
            handleNewKeyChange: this.onNewKeyChange,
            handleNewKeyFocus: this.onNewKeyFocus,
            handleNewValueBlur: this.onNewValueBlur,
            handleNewValueFocus: this.onNewValueFocus,
            handleNewValueKeyDown: this.onNewValueKeyDown,
            handleSortChange: this.handleSortChange,
            label,
            message,
            model,
            modelKeys,
            newKey,
            newValue,
            onCleaningSearchWord: this.handleCleaningSearchWord,
            onKeyBlur: this.handleKeyBlur,
            onRemoveKey: this.handleRemoveKey,
            onSearchChanged: this.handleSearchChanged,
            onValidityChange,
            onValueChange: this.handleValueChange,
            pristine,
            required,
            searchWord,
            showNewKeyValueRow,
            sorting,
            updateCellState: this.updateCellState,
            valid
        })}
        />;
    }
}

// eslint-enable-line max-lines
