import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../utils/ramda-utils';
import {findFirstFieldNameWithFalseValue} from '../utils/transform-utils';
import {xlMessageTypeProp} from '../xl-props/xl-message-type-prop';
import {externalModelToInternal, getExternalPropertyKeys, transformModelOnToggle} from './form-model-utils';
import {XlReactDipDynamicFormComponent} from './xl-react-dip-dynamic-form-component';
import {findCategoryNameByPropertyName, prepareGroupedProperties} from './xl-react-dip-dynamic-form-service';
import './xl-react-form.less';

export class XlReactDipDynamicForm extends Component {
    static propTypes = {
        allowLookupValues: PropTypes.bool,
        children: PropTypes.oneOfType([
            PropTypes.arrayOf(PropTypes.element),
            PropTypes.element
        ]),
        collapsed: PropTypes.bool,
        dateTimeFormat: PropTypes.string,
        disabled: PropTypes.bool,
        displayMode: PropTypes.string,
        handlers: PropTypes.object,
        message: xlMessageTypeProp,
        metadata: PropTypes.object,
        model: PropTypes.object,
        objLabel: PropTypes.string,
        onModelChange: PropTypes.func,
        onPristinityChange: PropTypes.func,
        onValidityChange: PropTypes.func,
        register: PropTypes.func,
        validators: PropTypes.arrayOf(PropTypes.func)
    };

    static defaultProps = {
        allowLookupValues: true,
        dateTimeFormat: 'YYYY-MM-DDTHH:mm:ss.SSSZ',
        handlers: {},
        message: {show: false},
        metadata: {},
        onModelChange: R.F,
        onPristinityChange: R.F,
        onValidityChange: R.F,
        validators: []
    };

    constructor(props) {
        super(props);
        this.batchChanges = {
            message: {},
            pristinity: {},
            validity: {}
        };
        this.state = {
            groupedProperties: prepareGroupedProperties(props.metadata, props.collapsed),
            message: {},
            pristinity: {},
            validity: {}
        };
        this.componentApis = {};
        this.externalProperties = getExternalPropertyKeys(this.props.model);
        this.resetPristinity();
        this.resetValidity();
    }

    componentWillMount() {
        RU.invoke('register', [this.api], this.props);
        RU.invoke('onPristinityChange', [this.state.pristinity], this.props);
        RU.invoke('onValidityChange', [this.state.validity], this.props);
    }

    componentWillReceiveProps(nextProps) {
        if (nextProps.metadata || nextProps.collapsed) {
            const isCollapsed = () => nextProps.collapsed || this.props.collapsed;
            const getMetadata = () => nextProps.metadata || this.props.metadata;
            this.setState({
                groupedProperties:
                    prepareGroupedProperties(getMetadata(), isCollapsed(), this.state.groupedProperties)
            });
        }
    }

    shouldComponentUpdate(nextProps, nextState) {
        const isNotEqual = (left, right, names) =>
            !R.reduce((acc, name) => acc && R.equals(left[name], right[name]), true, names);

        const statesToReRender = ['groupedProperties', 'validity'];
        const propsToReRender = [
            'collapsed',
            'disabled',
            'displayMode',
            'message',
            'metadata',
            'model',
            'objLabel'
        ];
        return isNotEqual(this.props, nextProps, propsToReRender) ||
            isNotEqual(this.state, nextState, statesToReRender);
    }

    handleLookupToggle = (ciPropName, isOn, value) => {
        if (isOn) {
            this.externalProperties = [...this.externalProperties, ciPropName];
        } else {
            this.externalProperties = R.filter(R.complement(R.equals(ciPropName)), this.externalProperties);
        }

        const model = transformModelOnToggle(this.props.model, ciPropName, isOn, this.memorized[ciPropName]);
        RU.invoke('onModelChange', [model], this.props);
        this.memorized[ciPropName] = value;
    };

    onChangeHandlerCreator = (ciPropName, statePropName, callbackName, value) => {
        this.batchChanges[statePropName] = {
            ...this.batchChanges[statePropName],
            [ciPropName]: value
        };
        const newState = {
            ...this.state[statePropName],
            ...this.batchChanges[statePropName]
        };
        const callback = () => {
            RU.invoke(callbackName, [newState], this.props);
            this.batchChanges[statePropName] = {};
        };
        this.setState({[statePropName]: newState}, callback);
    };

    onModelChangeHandlerCreator = (ciPropName) => (value) => {
        const withoutMessages = this.removeValidationMessages(this.props.model, ciPropName);
        const path = R.contains(ciPropName, this.externalProperties) ?
            ['external-properties', ciPropName] : [ciPropName];
        RU.invoke('onModelChange', [{...R.assocPath(path, value, withoutMessages)}], this.props);
    };

    onMessageChangeHandlerCreator = (ciPropName) => (value) => {
        this.onChangeHandlerCreator(ciPropName, 'message', 'onMessageChange', value);
    };

    onPristinityChangeHandlerCreator = (ciPropName) => (value) => {
        this.onChangeHandlerCreator(ciPropName, 'pristinity', 'onPristinityChange', value);
    };

    onValidityChangeHandlerCreator = (ciPropName) => (value) => {
        this.onChangeHandlerCreator(ciPropName, 'validity', 'onValidityChange', value);
    };

    removeValidationMessages = (model, propertyName) => {
        const validationMessages = model['validation-messages'];
        if (validationMessages) {
            const reduceFunc = (acc, validationMessage) =>
                validationMessage.property === propertyName ? acc : [...acc, validationMessage];
            return {
                ...model,
                'validation-messages': R.reduce(reduceFunc, [], validationMessages)
            };
        }
        return model;
    };

    api = {
        focusFieldByName: (fieldName) => {
            const isHiddenProperty = R.isNil(this.componentApis[fieldName]);
            if (fieldName && !isHiddenProperty) {
                this.componentApis[fieldName].focus();
            }
        },
        focusOnFirstInvalidField: () => {
            const nameOfInvalidField = findFirstFieldNameWithFalseValue(this.state.validity);
            if (nameOfInvalidField) {
                this.componentApis[nameOfInvalidField].focus();
                const categoryName = findCategoryNameByPropertyName(this.props.metadata, nameOfInvalidField);
                this.openCategoryGroup(categoryName);
            }
        },
        isDipFormPristine: () => R.isEmpty(R.filter((pr) => !pr, this.state.pristinity)),
        isDipFormValid: () => R.isEmpty(R.filter((pr) => !pr, this.state.validity)),
        resetPristinity: (config) => {
            R.forEach((api) => {
                api.resetPristinity(config);
            })(R.values(this.componentApis));
        }
    };

    memorized = {};

    openCategoryGroup = (categoryName) => {
        const transform = (item) => ({...item, isOpen: categoryName === item.category ? true : item.isOpen});
        this.setState({groupedProperties: R.map(transform, this.state.groupedProperties)});
    };

    resetPristinity() {
        R.forEach((metaProperty) => {
            this.state.pristinity[metaProperty.name] = true; // eslint-disable-line react/no-direct-mutation-state
        }, this.props.metadata.properties);
    }

    resetValidity() {
        R.forEach((metaProperty) => {
            this.state.validity[metaProperty.name] = true; // eslint-disable-line react/no-direct-mutation-state
        }, this.props.metadata.properties);
    }

    toggleCategoryGroup = (categoryName) => {
        const transform = (item) => ({...item, isOpen: categoryName === item.category ? !item.isOpen : item.isOpen});
        this.setState({groupedProperties: R.map(transform, this.state.groupedProperties)});
    };

    render() {
        const {
            allowLookupValues,
            children,
            collapsed,
            dateTimeFormat,
            disabled,
            displayMode,
            handlers,
            message,
            metadata,
            model,
            objLabel,
            validators
        } = this.props;

        const {groupedProperties} = this.state;
        const onRegister = (fieldName, api) => {
            this.componentApis[fieldName] = api;
        };

        return <XlReactDipDynamicFormComponent {...({
            allowLookupValues,
            children,
            collapsed,
            dateTimeFormat,
            disabled,
            displayMode,
            groupedProperties,
            handlers,
            message,
            metadata,
            model: externalModelToInternal(model),
            objLabel,
            onLookupToggle: this.handleLookupToggle,
            onMessageChangeHandlerCreator: this.onMessageChangeHandlerCreator,
            onModelChangeCreator: this.onModelChangeHandlerCreator,
            onPristinityChangeCreator: this.onPristinityChangeHandlerCreator,
            onRegister,
            onValidityChangeCreator: this.onValidityChangeHandlerCreator,
            pristinity: this.state.pristinity,
            toggleCategoryGroup: this.toggleCategoryGroup,
            validators,
            validity: this.state.validity
        })}
        />;
    }
}
