import R from 'ramda';
import React from 'react';
import {mount} from 'enzyme';
import {
    getDefaultValue,
    getModelValue,
    isComponentDisabled,
    isDisabled,
    printValue,
    resetPristinity,
    calculateMenuHeight,
    getScrollParent,
    shouldDisplayDropdownUp
} from '~/app/utils/components-utils';
import moment from 'moment';
import {XlReactWidgetAutocompleteInput} from '../../../app/xl-react-widget/xl-react-widget-autocomplete/components/xl-react-widget-autocomplete-input-component'; // eslint-disable-line max-len

describe('components-utils', () => {
    describe('getDefaultValue', () => {
        it('should return default value if it\'s defined', () => {
            expect(getDefaultValue()).toBeUndefined();
            expect(getDefaultValue({metadata: {}})).toBeUndefined();
            expect(getDefaultValue({metadata: {default: 1}})).toBe(1);
        });
    });
    describe('printValue', () => {
        it('should print null if value is not defined', () => {
            expect(printValue()).toBeNull();
        });

        it('should print properly moment object', () => {
            const value = moment('2010-10-20 4:30', 'YYYY-MM-DD HH:mm');
            expect(printValue(value)).toEqual('October 20th 2010, 4:30:00 am');
        });

        it('should print properly date object', () => {
            const value = moment('2010-10-20 4:30', 'YYYY-MM-DD HH:mm').toDate();
            expect(printValue(value)).toEqual('October 20th 2010, 4:30:00 am');
        });

        it('should print properly an object', () => {
            const value = {someKey: 'someValue'};
            expect(printValue(value)).toEqual('{"someKey":"someValue"}');
        });

        it('should print properly primitive types', () => {
            expect(printValue(1)).toEqual('1');
            expect(printValue('2')).toEqual('2');
            expect(printValue(true)).toEqual('true');
            expect(printValue(2.01)).toEqual('2.01');
        });
    });

    describe('getModelValue', () => {
        it('should get default value from options', () => {
            expect(getModelValue({metadata: {default: 4}})).toEqual(4);
        });

        it('should get default value from metadata', () => {
            expect(getModelValue({metadata: {default: 3}})).toEqual(3);
        });

        it('should get the default value first from options and if it doesn\'t specified in metadata', () => {
            expect(getModelValue({metadata: {default: 3}})).toEqual(3);
        });

        it('should get the value of specified property', () => {
            expect(getModelValue({name: 1}, 'name')).toEqual(1);
        });

        it('should get the value of specified property', () => {
            expect(getModelValue({
                metadata: {default: 3},
                name: 1
            }, 'name')).toEqual(1);
        });

        it('should get the default value if not found specified property', () => {
            expect(getModelValue({
                apple: 1,
                metadata: {default: 3}
            }, 'name')).toEqual(3);
        });

        it('should get fallback value if fetched value is empty string, empty object or null/undefined', () => {
            expect(getModelValue({apple: ''}, 'apple', 'fallbackValue')).toEqual('fallbackValue');
            expect(getModelValue({apple: {}}, 'apple', 'fallbackValue')).toEqual('fallbackValue');
            expect(getModelValue({apple: null}, 'apple', 'fallbackValue')).toEqual('fallbackValue');
            expect(getModelValue({apple: undefined}, 'apple', 'fallbackValue')).toEqual('fallbackValue');
            expect(getModelValue({}, 'apple', 'fallbackValue')).toEqual('fallbackValue');
        });
    });

    describe('isDisabled', () => {
        it('should return true if one of the parameters met the condition', () => {
            expect(isDisabled(true)).toEqual(true);
            expect(isDisabled(false, 'visual')).toEqual(true);
            expect(isDisabled(true, 'visual')).toEqual(true);
        });

        it('should return false if none of the parameters met the condition', () => {
            expect(isDisabled()).toEqual(false);
            expect(isDisabled(false, 'edit')).toEqual(false);
        });
    });

    describe('isComponentDisabled', () => {
        it('should return true if one of the parameters met the condition', () => {
            expect(isComponentDisabled({disabled: true})).toEqual(true);
            expect(isComponentDisabled({disabled: false, displayMode: 'visual'})).toEqual(true);
            expect(isComponentDisabled({disabled: true, displayMode: 'visual'})).toEqual(true);
        });

        it('should return false if none of the parameters met the condition', () => {
            expect(isComponentDisabled({})).toEqual(false);
            expect(isComponentDisabled({disabled: false, displayMode: 'edit'})).toEqual(false);
        });
    });

    describe('resetPristinity', () => {
        it('should update prisitinity state back to true', () => {
            const context = {setState: R.F, state: {pristine: false}};
            resetPristinity(context, {});
            expect(context.state).toEqual({pristine: true});
        });
    });

    describe('calculateMenuHeight', () => {
        const defaultProps = {
            menuHeight: 120,
            menuHeightWithSubActionHeight: 150,
            optionHeight: 23
        };

        it('should return default variations menuHeight', () => {
            expect(calculateMenuHeight(defaultProps.menuHeight, 40, defaultProps.optionHeight, false))
                .toBe(defaultProps.menuHeight);
            expect(calculateMenuHeight(defaultProps.menuHeight, 40, defaultProps.optionHeight, true))
                .toBe(defaultProps.menuHeightWithSubActionHeight);
        });

        it('should return calculation depends on suggestions length', () => {
            expect(calculateMenuHeight(defaultProps.menuHeight, 1, defaultProps.optionHeight, false)).toBe(23);
            expect(calculateMenuHeight(defaultProps.menuHeight, 1, defaultProps.optionHeight, true)).toBe(53);
        });
    });

    describe('getScrollParent', () => {
        const componentWrapper = mount(<XlReactWidgetAutocompleteInput/>);
        const inputWrapper = componentWrapper.find('input').getDOMNode();

        it('should have scrollable parent as documentElement', () => {
            expect(getScrollParent(inputWrapper)).toBe(document.documentElement);

            componentWrapper.getDOMNode().style.overflowY = 'inherit';
            expect(getScrollParent(inputWrapper)).toBe(document.documentElement);

            componentWrapper.getDOMNode().style.overflowY = 'visible';
            expect(getScrollParent(inputWrapper)).toBe(document.documentElement);

            componentWrapper.getDOMNode().style.overflowY = 'hidden';
            expect(getScrollParent(inputWrapper)).toBe(document.documentElement);
        });

        it('should not have scrollable parent as documentElement', () => {
            componentWrapper.getDOMNode().style.overflowY = 'auto';
            expect(getScrollParent(inputWrapper)).toBe(componentWrapper.getDOMNode());

            componentWrapper.getDOMNode().style.overflowY = 'scroll';
            expect(getScrollParent(inputWrapper)).toBe(componentWrapper.getDOMNode());
        });

        it('should return undefined', () => {
            expect(getScrollParent(null)).toBe(undefined);
            expect(getScrollParent(undefined)).toBe(undefined);
        });
    });
    describe('shouldDisplayDropdownUp', () => {
        const componentWrapper = mount(<XlReactWidgetAutocompleteInput/>);
        const inputWrapper = componentWrapper.find('input').getDOMNode();

        it('when input exists, should return false', () => {
            expect(shouldDisplayDropdownUp(inputWrapper, 120)).toBe(false);
        });

        it('when input does not exist, should return false', () => {
            expect(shouldDisplayDropdownUp(null, 120)).toBe(false);
        });
    });
});
