import R from 'ramda';
import {
    getInvalidCategoryNames,
    groupByCategory
} from '~/app/xl-react-form/xl-react-dip-dynamic-form-service';

import invalidCommonCategoryModel from './invalid-common-category-model.json';
import allInvalidCategoriesModel from './all-invalid-categories-model.json';
import noInvalidCategoriesModel from './no-invalid-categories-model.json';

describe('XlReactDipDynamicFormServiceTest', () => {
    it('should sort by category name insensitive to upper case, but having Common in precedence as an exception ' +
        'case', () => {
        const result = groupByCategory([
            {category: 'water'},
            {category: 'Common'},
            {category: 'Common'},
            {category: 'Milk'}
        ]);
        expect(R.map(R.prop('category'), result)).toEqual(['Common', 'Milk', 'water']);
    });

    it('should default property categories to "Common" if they are not defined', () => {
        const result = groupByCategory([
            {name: 'A'},
            {name: 'B'},
            {category: 'Other', name: 'C'}
        ]);
        expect(R.map(R.prop('category'), result)).toEqual(['Common', 'Other']);
        expect(R.find(R.propEq('category', 'Common'), result).properties).toEqual([
            {category: 'Common', name: 'A'},
            {category: 'Common', name: 'B'}
        ]);
    });

    it('should group properties by category', () => {
        const result = groupByCategory([
            {category: 'One', name: 'A'},
            {name: 'B'},
            {category: 'Three', name: 'C'},
            {category: 'One', name: 'D'},
            {category: 'Common', name: 'E'}
        ]);
        expect(R.map(R.prop('category'), result)).toEqual(['Common', 'One', 'Three']);
        expect(R.find(R.propEq('category', 'Common'), result).properties).toEqual([
            {category: 'Common', name: 'B'},
            {category: 'Common', name: 'E'}
        ]);
        expect(R.find(R.propEq('category', 'One'), result).properties).toEqual([
            {category: 'One', name: 'A'},
            {category: 'One', name: 'D'}
        ]);
        expect(R.find(R.propEq('category', 'Three'), result).properties).toEqual([
            {category: 'Three', name: 'C'}
        ]);
    });

    describe('open property', () => {
        it('should set "open" property to "true" if collapsed parameter is null', () => {
            const result = groupByCategory([{category: 'Common'}], null);
            expect(R.nth(0, result).isOpen).toBeTruthy();
        });

        it('should set "open" property to "true" if collapsed parameter is undefined', () => {
            const result = groupByCategory([{category: 'Common'}], undefined);
            expect(R.nth(0, result).isOpen).toBeTruthy();
        });

        it('should set "open" property to what is specified in collapsed parameter', () => {
            const result = groupByCategory([{category: 'Common'}], false);
            expect(R.nth(0, result).isOpen).toBeFalsy();
        });

        it('should set "open" property to "false" when collapse is "auto" and no VISIBLE REQUIRED fields', () => {
            const result = groupByCategory([
                {category: 'Common', name: 'A'},
                {category: 'Common', name: 'B'}
            ], 'auto');
            expect(R.nth(0, result).isOpen).toBeFalsy();
        });

        const autoCollapseTestPipe = R.pipe(
            R.curry(groupByCategory)(R.__, 'auto', undefined),
            R.nth(0),
            R.prop('isOpen')
        );

        it('should set "open" property to "true" when collapse is "auto" and there are REQUIRED fields', () => {
            expect(autoCollapseTestPipe([
                {category: 'Common', name: 'A', required: true},
                {category: 'Common', name: 'B'}
            ])).toBeTruthy();

            expect(autoCollapseTestPipe([
                {category: 'Common', hidden: true, name: 'A', required: true},
                {category: 'Common', name: 'B', required: true}
            ])).toBeTruthy();
        });

        it('should set "open" property to "false" when collapse is "auto" and there are REQUIRED fields but they ' +
            'all are hidden', () => {
            expect(autoCollapseTestPipe([
                {category: 'Common', hidden: true, name: 'A', required: true},
                {category: 'Common', hidden: true, name: 'B', required: true}
            ])).toBeFalsy();
        });

        it('should set "open" property from old group properties', () => {
            const result = groupByCategory([
                {category: 'Common', name: 'A'},
                {category: 'Uncommon', name: 'B'}
            ], undefined, [
                {category: 'Common', isOpen: false, name: 'A'},
                {category: 'Uncommon', isOpen: true, name: 'B'}
            ]);
            expect(R.nth(0, result).isOpen).toBeFalsy();
            expect(R.nth(1, result).isOpen).toBeTruthy();
        });
    });

    describe('getInvalidCategoryNames', () => {
        it('should return the array of names of categories where at least one property is invalid', () => {
            expect(getInvalidCategoryNames(invalidCommonCategoryModel)).toEqual(['Common']);
            expect(getInvalidCategoryNames(allInvalidCategoriesModel)).toEqual(['Common', 'Personal Data']);
            expect(getInvalidCategoryNames(noInvalidCategoriesModel)).toEqual([]);
        });
    });
});
